/*
 * cntable.cc: caf neighbor table
 */
#include <caf/cntable.h>

CAFnTable::CAFnTable() {
  list_start = NULL;  // empty
  tablen = 0;
}

void
CAFnTable::AddEntry(const cntable_ent &ent) {

    cntable_ent *nl;
 
    /*
     * verify if the node already there
     */
 
    for (nl = list_start; nl; nl = nl->next) {
        if (nl->nid == ent.nid) break;
    }
 
    if (nl) {
        // copy the content

      nl->cid = ent.cid;
      nl->state = ent.state;
      nl->enlt = ent.enlt;
      nl->stime = ent.stime;
      nl->nlen = ent.nlen;

      if (nl->nlen > 0) {
	// copy the neighbor list
	bcopy(ent.nlist, nl->nlist, sizeof(u_int32_t)*nl->nlen);
      }

    } else {
      
      nl = new cntable_ent;

      nl->nid = ent.nid;
      nl->cid = ent.cid;      
      nl->enlt = ent.enlt;  
      nl->stime = ent.stime;
      nl->state = ent.state;  
      nl->nlen = ent.nlen;
      
      if (nl->nlen > 0) {
	// copy the neighbor list
	bcopy(ent.nlist, nl->nlist, sizeof(u_int32_t)*nl->nlen);
      }

      // insert it into the list
      nl->next = list_start;
      list_start = nl;

      tablen++;
    }
}

cntable_ent *
CAFnTable::FirstEntry () {

  crtp = list_start;
  return crtp;
}

cntable_ent *
CAFnTable::NextEntry () {
  if (crtp) {
    crtp = crtp->next;
    return crtp;
  }
  return NULL;
}

cntable_ent *
CAFnTable::GetEntry (u_int32_t nodeid) {

    cntable_ent *nl;
 
    /*
     * verify if the node already there
     */
 
    for (nl = list_start; nl; nl = nl->next) {
      if (nl->nid == nodeid) {
	return nl;
      }
    }
    return NULL;

}

u_int32_t 
CAFnTable::tablesize () {
  
  return tablen;

}

void
CAFnTable::cleanTable () {
    cntable_ent *lp, *nl;
    
    lp = list_start;

    while (lp) {
      nl = lp->next;
      delete lp;
      lp = nl;
    }

    list_start = NULL;
    tablen = 0;
}
