
#include <stdlib.h>
 
#include "random.h"
#include "trafgen.h"
#include "ranvar.h"

#define INTERVAL1290_1 0.0002
#define INTERVAL1290_2 0.0003
/* 
 * Mp3 traffic source based on switched ethernet traffic of SunRay 
 */

class MP3_Traffic : public TrafficGenerator {
 public:
	MP3_Traffic();
	virtual double next_interval(int&);
	//HACK so that udp agent knows interpacket arrival time within a burst
	inline double interval() { return (interval_); }
 protected:
	virtual void start();
	void init();
	double rate_;     /* send rate during on time (bps) */
	double interval_; /* packet inter-arrival time during burst (sec) */
	double random_;
	int seqno_;
	int maxpkts_;
	int w802_;
	int w1290_;
	int maxlen1290_;
	int maxlen802_;
	int cur_run_;
	int w0_cnt_;
        int len802[8];
        int len802_2[5];
        int w802_cnt1_;
};


static class MP3TrafficClass : public TclClass {
 public:
	MP3TrafficClass() : TclClass("Application/Traffic/MP3") {}
	TclObject* create(int, const char*const*) {
		return (new MP3_Traffic());
	}
} class_mp3_traffic;

MP3_Traffic::MP3_Traffic() : seqno_(0)
{
	//bind_bw("rate_", &rate_);
	//bind("random_", &random_);
	//bind("packetSize_", &size_);
	//bind("maxpkts_", &maxpkts_);
}

void MP3_Traffic::init()
{

	// initial size = 802 and interval = 0.00011
	// how many 802-lengh packets

	size_ = 1290;
	interval_ = (double)Random::uniform(INTERVAL1290_1,INTERVAL1290_2);
	maxlen1290_ = (int)Random::uniform(10, 20);
	cur_run_ = 1290;

	maxlen802_ = (int)Random::uniform(1, 10);

	// define weight

	w802_ = 8;
	w1290_ = 11;

	w0_cnt_ = 0;
	w802_cnt1_ = 0;
	// define different len

	len802[0] = 1498;
	len802[1] = 1498;
	len802_2[0] = 1326;
	len802_2[1] = 1110;
	len802_2[2] = 1170;
	len802_2[3] = 1378;
	len802_2[4] = 1230;
	len802[6] = 130;
	len802[7] = 1426;


	if (agent_)
		agent_->set_pkttype(PT_CBR);
}

void MP3_Traffic::start()
{
        init();
        running_ = 1;
        timeout();
}

double MP3_Traffic::next_interval(int& size)
{
  int i;
	switch (cur_run_) {
	  case 802:

	    if (w802_cnt1_ < 2 || w802_cnt1_ > 5) { 
	        size_ = len802[w802_cnt1_];
	
	    } else {
	        i = (int)Random::uniform(0,4);

	        size_=len802_2[i];
	     
	    }

            w802_cnt1_++;
	    if (w802_cnt1_ == 8) w802_cnt1_ = 0; 
	    
	    interval_ = (double)Random::uniform(0.0001, 0.001);

	    w802_--;
	    maxlen802_--;

	    // decide the max pkts of next time 802 sequence.
	    // Use WRR so that if w802 is positive, more credits
	    // otherwise, less credits.

	    if (maxlen802_ == 0) {
		
		// try to balance the weight, allow 10% wave

		w802_ = 8 + w802_;
		maxlen802_ = (int)(w802_ * (1 + Random::uniform(-0.3,0.3)));
		
		// pass to 1290 packets
		cur_run_ = 1290;
		
		// change packetsize and interval
		size_ = 1290;
		interval_ = (double) Random::uniform(0.001,0.01);
		
		
	    }
	    
	    break;

	  case 1290:

	    size_ = 1290;
	    interval_ = (double)Random::uniform(INTERVAL1290_1,INTERVAL1290_2);
	    
	    w1290_--;
	    maxlen1290_--;

	    if (maxlen1290_ == 0) {
		
		// try to balance the weight, allow 10% wave

		w1290_ = 11 + w1290_;
		maxlen1290_ = (int)(w1290_ * (1 + Random::uniform(-0.3,0.3)));
	
		w0_cnt_++;

		if (w0_cnt_ > 100) {
		
		    // pass to random packets
		    cur_run_ = 0;
		    w0_cnt_ = 0;

		    // change packetsize and interval
		    size_ = (int)(Random::uniform(50,1500));
		    interval_ = (double)Random::uniform(0.001,0.01);
		} else {
	
		    cur_run_ = 802;
	            // change packetsize and interval
                    size_ = 1498;
                    interval_ = 0.01;                    

		}
	    }
	    break;
	    
	  case 0:

	    // simply switch to 1290 packets
	    cur_run_ = 1290;
	    size_ = 1290;
	    interval_ = 0.001;
	    break;
	    
	  default:

	    printf("Error in mp3 traffic generator\n");
	    break;
	}
	
	// Recompute interval in case rate_ or size_ has changes

	// interval_ = (double)(size_ << 3)/(double)rate_;

	double t = interval_;

	// if (random_)
	// 	t += interval_ * Random::uniform(-0.5, 0.5);	
	// size = size_;
	//if (++seqno_ < maxpkts_)
	//	return(t);
	//else
	//	return(-1);
	size = size_;
	return(t);
}


