
#include <stdlib.h>
 
#include "random.h"
#include "trafgen.h"
#include "ranvar.h"

/* 
 * Web traffic source based on switched ethernet traffic 
 */

class WEB_Traffic : public TrafficGenerator {
 public:
	WEB_Traffic();
	virtual double next_interval(int&);
	//HACK so that udp agent knows interpacket arrival time within a burst
	inline double interval() { return (interval_); }
 protected:
	virtual void start();
	void init();
	double ontime_;  /* average length of burst (sec) of a page*/
	double offtime_; /* average idle period (sec) of a page */
	double interval_; /* inter-packet time at burst rate */
	double burstlen_; /* average # packets/burst */
	double shape_;    /* pareto shape parameter */
	unsigned int rem_; /* number of packets remaining in current burst of a page */
	double p1_;       /* parameter for pareto distribution to compute
			   * number of packets in burst.
		           */
	double p2_;       /* parameter for pareto distribution to compute
		     	   * length of idle period.
		     	   */
        double pontime_;  /* average length of burst (sec) of inside a page */
        double pofftime_; /* average idle period (sec) inside a page */
        double pburstlen_; /* average # packets/burst */
        double pshape_;    /* pareto shape parameter */
        unsigned int prem_; /* number of packets remaining in current burst inside a  page */
        double pp1_;       /* parameter for pareto distribution to compute
                           * number of packets in burst.
                           */
        double pp2_;       /* parameter for pareto distribution to compute
                           * length of idle period.
                           */            


	int on_;          /* denotes whether in the on or off state */
	int w900_;
	int w400_;
	int maxlen900_;
	int maxlen400_;
	int cur_run_;
        int t400;
        int bnpkt,inpkt;
};


static class WEBTrafficClass : public TclClass {
 public:
	WEBTrafficClass() : TclClass("Application/Traffic/WEB") {}
	TclObject* create(int, const char*const*) {
		return (new WEB_Traffic());
	}
} class_web_traffic;

WEB_Traffic::WEB_Traffic() 
{
	// all parameters are for inter-pages
	bind_time("burst_time_", &ontime_);
	bind_time("idle_time_", &offtime_);
	bind("shape_", &shape_);
	bind_time("pb_time_", &pontime_);
	bind_time("pi_time_",&pofftime_);
	bind("pshape_",&pshape_);

	//bind("maxpkts_", &maxpkts_);
}

void WEB_Traffic::init()
{
	// packet size
	size_ = (int)Random::uniform(800,1498);
	// interval: aggregated data rate 0.6M
	interval_ = 0.02;

	burstlen_ = ontime_/interval_;

	on_  = 0;
	rem_ = 0;
	prem_ = 0;

	p1_ = burstlen_ * (shape_ - 1.0)/shape_;
	p2_ = offtime_ * (shape_ - 1.0)/shape_;

	w900_ = 45;
	w400_ = 8;
	t400 = 0;

	maxlen900_ = (int)Random::uniform(10, 40);
	maxlen400_ = (int)Random::uniform(1, 10);

	cur_run_ = 900;

	// decide on/off within a page

	// according to maximum 70M rate
	interval_ = (double)Random::uniform(0.0001,0.0002);

	pburstlen_ = pontime_/interval_;

	pp1_= pburstlen_ * (pshape_ - 1.0)/pshape_; 
	pp2_= pofftime_ * (pshape_ - 1.0)/pshape_; 

	if (agent_)
		agent_->set_pkttype(PT_CBR);
}

void WEB_Traffic::start()
{
        init();
        running_ = 1;
        timeout();
	bnpkt = (int) Random::uniform(8,30);
	inpkt = (int) Random::uniform(2,10);
}

double WEB_Traffic::next_interval(int& size)
{


	double t = interval_;

	on_ = 1;
	
	if (rem_ == 0) {

		// idle time: for inter-page sleeping 
		/* compute # of packets in next burst */
		rem_ = int(Random::pareto(p1_, shape_) + .5);

		if (rem_ == 0) rem_ = 1;

		/* computer idle time */
		
		t += Random::pareto(p2_, shape_);
		on_ = 0;

	} else {

		// burst time: intra-page behavior

		rem_--;

		// pkt size distribution

		switch (cur_run_) {

		case 900:
	
		    size_ = (int)Random::uniform(800,1500);
		    w900_--;
		    maxlen900_--;
		    if (maxlen900_ == 0) {
			w900_ = 45+w900_;
			maxlen900_ = (int)(w900_*(1 + Random::uniform(-0.2,0.2)));
			t400++;
			if (t400 > 10) {
			    cur_run_ = 400;
			    size_ = (int)Random::uniform(63,800);
			    t400 = 0;
			}
		    }		    
		    break;
		case 400:
		  
		    size_ = (int)Random::uniform(63,800);
                    w400_--;
                    maxlen400_--;
                    if (maxlen400_ == 0) {
                        w400_ = 8+w400_;
                        maxlen400_ = (int)(w400_*(1 + Random::uniform(-0.1,0.1)));
                        cur_run_ = 900;
                        size_ = (int)Random::uniform(800,1500);
                    }
                    break;        
		default:
		    printf("Error in web traffic\n");

		}		

		// interval_ distribution inside a page.

        	if (prem_ == 0) {
 
                    // idle time: for inter-page sleeping
                    /* compute # of packets in next burst */
                    prem_ = int(Random::pareto(pp1_, pshape_) + .5);
 
                    if (prem_ == 0) prem_ = 1;
 
                    /* computer idle time */
 
                    t += Random::pareto(pp2_, pshape_);
		    
		    bnpkt = (int) Random::uniform(8,15);
		    inpkt = (int) Random::uniform(2,10);
		} else {
		    /* create bursty of 10 - 15 packets */
		    
		    
		    if (bnpkt > 0) {
		       t = (double)Random::uniform(0.0001,0.0002);
		       bnpkt --;
		    } else {
		      if (inpkt > 0) {
			  t = (double)Random::uniform(0.001,0.01);
			  inpkt --;
		      } else {
		          bnpkt = (int) Random::uniform(8,30);
			  inpkt = (int) Random::uniform(2,10);
		          t = (double)Random::uniform(0.001,0.01);
		      }
		    }
		    prem_--;
		}	
		

	}
	size = size_;

	return(t);

}


